# $Id$

package ObsData::Repository;

use strict;
use warnings;
use ObsData;
use DBD::SQLite;

our @ISA = qw(ObsData);

my @loglevel = (
    'DEBUG',
    'INFO',
    'RESULT',
    'WARNING',
    'ERROR',
    'FATAL',
);

=head1 METHODS

=head2 new

Parameters list:

=item configfile

The configuration file to use

=item sublog

Coderef to external loging function

=item subloglevel

Minimum log level to call sublog, default is 1

=item minloglevel

Minimum loglevel for internal logging

=item dir

The directory where internal data are put

=cut

sub new {
    my ($class, %config) = @_;

    my $odr = $class->SUPER::new($config{configfile}) or return undef;

    $odr->{sublog} = $config{sublog};
    $odr->{subloglevel} = defined($config{subloglevel}) ? $config{subloglevel} : 1;
    $odr->{minloglevel} = defined($config{minloglevel}) ? $config{minloglevel} : 1;

    if(!($config{dir} && -d $config{dir})) {
        return undef;
    }

    open($odr->{loghandle}, "> $config{dir}/logs/obsdata.log") or return undef; 
    
    bless($odr, $class);
}

=head2 logmsg

Add a message to log stack

Parameters list:

=item loglevel 

The log level of message (num or text)

=item info

hash ref to additionnal info

=item message ...

printf like args

=cut

sub logmsg {
    my ($self, $level, $info, $pf, @pfa) = @_;
    my $textlevel = $level =~ /^\d+$/ ? $loglevel[$level] : $level;
    my $nlevel = logleveln($textlevel);
    
    my @call = caller;
    
    $textlevel ||= "";
    if (!defined($nlevel)) {
        $textlevel = 'INFO';
        $nlevel = logleveln($textlevel);
        $self->logmsg('ERROR', undef,
           'Invalid use of loglevel "%s" at %s line %d, unsing %s"',
           $level || "", $call[1], $call[2], $textlevel);
    }

    if ($self->{minloglevel} <= $nlevel) {
       my $h = $self->{loghandle};
       print $h "[$textlevel]: " . sprintf($pf, @pfa) . "\n";
    }

    if ($self->{sublog} && $self->{subloglevel} <= $nlevel) {
       $self->{sublog}->($textlevel, $info, sprintf($pf, @pfa));
    }

}

sub logleveln {
    my ($t) = @_;
    my $i = 0;
    foreach (@loglevel) {
        return $i if ($t eq $_);
        $i++
    }
    return undef;
}

sub init_repository {
    my ($dir) = @_;

    if (!-d $dir) {
        mkdir $dir or return 0;
    }

    if (!-d "$dir/logs") {
        mkdir "$dir/logs" or return 0;
    } 

    my $db = DBI->connect("dbi:SQLite:dbname=$dir/obsdata.db","","",
        { RaiseError => 1, AutoCommit => 1}
    ) or return 0;

    my $odr = ObsData::Repository->new(dir => $dir);
    $odr->logmsg('INFO', undef, "Repository succefully created in %s", $dir);

    1;
}

1;
